@@ -47,6 +47,10 @@
 #include <map>
 #include <random>
 
+#ifdef HAVE_LIBEV
+#  include <ev.h>
+#endif // HAVE_LIBEV
+
 #include "url-parser/url_parser.h"
 
 #include "template.h"
@@ -106,9 +110,6 @@ std::string percent_encode(const unsigned char *target, size_t len);
 
 std::string percent_encode(const std::string &target);
 
-// percent-encode path component of URI |s|.
-std::string percent_encode_path(const std::string &s);
-
 template <typename InputIt>
 std::string percent_decode(InputIt first, InputIt last) {
   std::string result;
@@ -566,6 +567,12 @@ std::string to_numeric_addr(const struct sockaddr *sa, socklen_t salen);
 // Sets |port| to |addr|.
 void set_port(Address &addr, uint16_t port);
 
+// Get port from |su|.
+uint16_t get_port(const sockaddr_union *su);
+
+// Returns true if |port| is prohibited as a QUIC client port.
+bool quic_prohibited_port(uint16_t port);
+
 // Returns ASCII dump of |data| of length |len|.  Only ASCII printable
 // characters are preserved.  Other characters are replaced with ".".
 std::string ascii_dump(const uint8_t *data, size_t len);
@@ -698,6 +705,17 @@ template <typename Clock, typename Rep> Rep clock_precision() {
   return duration.count();
 }
 
+#ifdef HAVE_LIBEV
+template <typename Duration = std::chrono::steady_clock::duration>
+Duration duration_from(ev_tstamp d) {
+  return std::chrono::duration_cast<Duration>(std::chrono::duration<double>(d));
+}
+
+template <typename Duration> ev_tstamp ev_tstamp_from(const Duration &d) {
+  return std::chrono::duration<double>(d).count();
+}
+#endif // HAVE_LIBEV
+
 int make_socket_closeonexec(int fd);
 int make_socket_nonblocking(int fd);
 int make_socket_nodelay(int fd);
@@ -828,8 +846,10 @@ StringRef make_http_hostport(OutputIt first, const StringRef &host,
   return StringRef{first, p};
 }
 
-// Dumps |src| of length |len| in the format similar to `hexdump -C`.
-void hexdump(FILE *out, const uint8_t *src, size_t len);
+// hexdump dumps |data| of length |datalen| in the format similar to
+// hexdump(1) with -C option.  This function returns 0 if it succeeds,
+// or -1.
+int hexdump(FILE *out, const void *data, size_t datalen);
 
 // Copies 2 byte unsigned integer |n| in host byte order to |buf| in
 // network byte order.
@@ -875,7 +895,7 @@ OutputIt random_alpha_digit(OutputIt first, OutputIt last, Generator &gen) {
 // Fills random bytes to the range [|first|, |last|).
 template <typename OutputIt, typename Generator>
 void random_bytes(OutputIt first, OutputIt last, Generator &gen) {
-  std::uniform_int_distribution<> dis(0, 255);
+  std::uniform_int_distribution<uint8_t> dis;
   std::generate(first, last, [&dis, &gen]() { return dis(gen); });
 }
 
@@ -945,9 +965,11 @@ StringRef rstrip(BlockAllocator &balloc, const StringRef &s);
 #ifdef ENABLE_HTTP3
 int msghdr_get_local_addr(Address &dest, msghdr *msg, int family);
 
-unsigned int msghdr_get_ecn(msghdr *msg, int family);
+uint8_t msghdr_get_ecn(msghdr *msg, int family);
 
-int fd_set_send_ecn(int fd, int family, unsigned int ecn);
+// msghdr_get_udp_gro returns UDP_GRO value from |msg|.  If UDP_GRO is
+// not found, or UDP_GRO is not supported, this function returns 0.
+size_t msghdr_get_udp_gro(msghdr *msg);
 #endif // ENABLE_HTTP3
 
 } // namespace util
